import {Legend, Anchor, Point4Anchor} from './DiagramModel';
import {Point, BackgroundPaniter, ViewTool} from './UIUtils';
import {Editor, Selection} from './Editor';

/**
 * 图例编辑器
 * @author zhaoyk
 */
export class LegendEditor extends Editor{

	legendSize = 300;

	legendData: Array<Legend>;
	legendViews: Object;

	private pointsForAnchor: Array<Point4Anchor>;
	private focusP4A: Point4Anchor;
	private focusAnchor: Anchor;

	protected initCanvasSize(): Point {
		return new Point(this.container.clientWidth, this.container.clientHeight);
	}
	
	addListeners(): void {
		this.canvas.addEventListener('click', event=>{this.onClick(event)});
		this.container.addEventListener("mousemove", event=>{this.onMouseMove(event)});
	}
	
	showData(legendData?: Array<Legend>): void {
		this.legendData = legendData;
		this.legendViews = new Object();
		
		this.selection = new Selection();
		
		this.setState(this.getState());

		this.redraw();
	}
	
	setState(state: number): void{
		this.state = state;
		this.pointsForAnchor = null;
		
		if(this.state == 8) { //设置锚点
			const legend: Legend = this.getSelComp();
			
			this.pointsForAnchor = Legend.getPointsForAnchor(legend, this.legendViews[legend.id].legendLocation);
			
			this.redraw();
		} else if (this.state == 9) { //绘制图例 
			
		}
		//TODO
	}

	redraw():void {
		this.ctx.clearRect(-1, -1, this.canvas.width + 1, this.canvas.height + 1);  
		
		const col = Math.floor(this.canvasSize.x / this.legendSize);
		const row = Math.floor(this.legendData.length / col) + 1;
		const h1 = row * this.legendSize + 10;
		if(h1 > this.canvasSize.y) {
			this.canvas.height = h1; // 实际渲染像素
			this.canvas.style.height = h1 + 'px'; // 控制显示大小
			
			this.ctx.translate(0.5, 0.5);
		}
		
		this.bgPainter.paint();
		
		//layout 
		for(var r = 0; r < row; r++){
			for(var c = 0; c < col; c++) {
				const idx = r * col + c;
				if(idx >= this.legendData.length)
					break;
				
				const legend = this.legendData[idx];
				const point = new Point(c * this.legendSize, r * this.legendSize);
				const lv = new LegendView();
				lv.viewLocation = point;
				lv.legendLocation = new Point(point.x + this.margin, point.y + this.margin);;
				this.legendViews[legend.id] = lv;
				
				if(legend.anchors && legend.anchors.length > 0) {
					const p4As = Legend.getPointsForAnchor(legend, lv.legendLocation);
					for(const p of p4As) {
						if(p.anchor)
							lv.anchorPoints[p.anchor.code] = p.point;
					}
				}
			}
		}
		
		for(const legend of this.legendData){
			this.drawLegend(legend, this.legendViews[legend.id]);
		}
		
	}
	
	private drawLegend(legend: Legend, legendView: LegendView){		
		this.ctx.setLineDash([]);
		const selected = this.getSelComp() == legend;
		if(selected){
			this.ctx.lineWidth = 2;	
			this.ctx.strokeStyle = "#5783FA";
		} else {
			this.ctx.lineWidth = 1;	
			this.ctx.strokeStyle = "#9b9ea3";
		}
		
		this.ctx.beginPath();
		const l1 = legendView.viewLocation;
		this.ctx.rect(l1.x ,l1.y, this.legendSize, this.legendSize);
		this.ctx.stroke();
		
		this.ctx.beginPath();
		this.ctx.lineWidth = 1;
		this.ctx.strokeStyle = "#000000";
		const l2 = legendView.legendLocation;
		this.ctx.rect(l2.x, l2.y, legend.width, legend.height);
		this.ctx.stroke();
		
		var info = legend.code;
		if(legend.name)
			info += '  ' + legend.name;
		info += '(' + legend.width + '*'  + legend.height + ')';
		if(legend.equipmentTypes && legend.equipmentTypes.length > 0)
			info += '\n[' + legend.equipmentTypes + ']';
		if(legend.dataFilter)
			info += ViewTool.filterToStr(legend.dataFilter);
		if(legend.remark && legend.remark.length > 0)
			info += '\n' + legend.remark;
		
		if(legend.iconId)
			info += '\n*图标已上传';

		ViewTool.drawText(this.ctx, info, new Point(l2.x, l2.y + legend.height + 5), new Point(this.legendSize - 20, 0));
		
		var color;
		if(this.pointsForAnchor && selected){ //设置锚点
			for(const p of this.pointsForAnchor) {
				this.ctx.beginPath();
				
				color = "#9b9ea3";
				var fill = false;
				if(p.selected) {
					if(p.anchor && p.anchor.multiple)
						color = "#00aa7f";
					else
						color = "#000000";
					fill = true;
				} else if(this.focusP4A && p.code == this.focusP4A.code) {
					fill = true;
				} 
				ViewTool.paintPoint(this.ctx, p.point, color, fill, 3);
			}
		} else { //绘制图例的锚点
			if(legend.anchors && legend.anchors.length > 0) {
				for(const a of legend.anchors) {
					const r = this.focusAnchor == a ? 4 : 3;
					color = a.multiple ? "#00aa7f" : "#000000";
					this.ctx.beginPath();
					ViewTool.paintPoint(this.ctx, legendView.anchorPoints[a.code], color, true, r);
				}
			}
		}
	}
	
	private onClick(event): void {
		if(this.state == 8) { //anchor
			if(this.focusP4A) {
				this.focusP4A.selected = !this.focusP4A.selected;
				this.redraw();
			}
		} else {
			if(this.focusAnchor) { //编辑锚点
				this.editorPart.editAnchor(this.focusAnchor);
			} else { //selection
				var sel:any = this.findLegend(event.layerX, event.layerY);
					
				if(sel != this.getSelComp)
					this.setSelComp(sel);	
			}
		}
	}
	
	findLegend(x:number, y:number): Legend {
		for(const l of this.legendData){
			const point = this.legendViews[l.id].viewLocation;
			if(x >= point.x && y >= point.y && x <= point.x + this.legendSize && y <= point.y + this.legendSize)
				return l;
		}
		return null;
	}
	
	setSelComp(comp: any): void{
		this.selection.setSel(comp);
		this.redraw();
		
		this.editorPart.onSelectionChange();
	}
	
	getSelComp(): any {
		if(!this.selection)
			return null;
		return this.selection.getSel();
	}
	
	buildSelInfo(){
		var info = null;

		const sel = this.getSelComp();
		if(sel != null) {
			//TODO
		}
		
		if(info == null)
			info = '~';
		return info;
	}
	
	getState(): number {
		const sel = this.getSelComp();
		return sel != null ? 1 : 0;
	}
	
	addLegend(legendCode:string): Legend {
		const l = new Legend();
		l.code = legendCode;
		l.width = 200;
		l.height = 200;
		this.legendData.push(l);
		this.redraw();
		
		return l;
	}
	
	storeAnchors(): void{
		if(this.pointsForAnchor){
			const arr = new Array<Anchor>();
			for(const p of this.pointsForAnchor) {
				if(p.selected){
					if(p.anchor)
						arr.push(p.anchor);
					else {
						const a = new Anchor();
						a.code = p.code;
						arr.push(a);
					}
				}
			}
			this.getSelComp().anchors = arr;
			console.log(this.getSelComp());
		}
	}
	
	private onMouseMove(event): void {
		const x = event.layerX;
		const y = event.layerY;
		if(this.state == 8) { //设置锚点
			var focus: Point4Anchor = null;
			for(const p of this.pointsForAnchor) {
				const p1 = new Point(p.point.x - 3, p.point.y - 3);
				const p2 = new Point(p.point.x + 3, p.point.y + 3);
				if(x >= p1.x && y >= p1.y && x <= p2.x && y <= p2.y) {
					focus = p;
					break;
				}
			}
			if(focus != this.focusP4A) {
				this.focusP4A = focus;
				this.redraw();
			}
		} else if(this.state == 9) { //绘图
			//TODO
			this.redraw();
		} else {
			//高亮显示锚点
			var fa: Anchor = null;
			const legend:Legend = this.getSelComp();
			if(legend && legend.anchors){
				for(const a of legend.anchors) {
					const p:Point = this.legendViews[legend.id].anchorPoints[a.code];
					const p1 = new Point(p.x - 3, p.y - 3);
					const p2 = new Point(p.x + 3, p.y + 3);
					if(x >= p1.x && y >= p1.y && x <= p2.x && y <= p2.y) {
						fa = a;
						break;
					}
				}
			}
			if(this.focusAnchor != fa) {
				this.focusAnchor = fa;
				this.redraw();
			}
			
			var info = '';
			if(this.focusAnchor) {
				info = this.focusAnchor.code;
				if(this.focusAnchor.name)
					info += '    ' + this.focusAnchor.name;
				if(this.focusAnchor.multiple)
					info += '    动态数量';
				if(this.focusAnchor.acceptRelations && this.focusAnchor.acceptRelations.length > 0)
					info += '    ' + this.focusAnchor.acceptRelations;
				if(this.focusAnchor.toEquipmentTypes && this.focusAnchor.toEquipmentTypes.length > 0){
					info += '  --  ' + this.focusAnchor.toEquipmentTypes;
					if(this.focusAnchor.toDataFilter)
						info += ViewTool.filterToStr(this.focusAnchor.toDataFilter);
				}
			}
			this.editorPart.dynInfo = info;
		}
	}
	
	private onKeydown(event): void {
		//if(event.keyCode)
		console.log(event.keyCode);
	}

}

class LegendView {
	
	viewLocation:Point;
	
	legendLocation:Point;
	
	anchorPoints:Object = new Object();
	
}
